@@ -1,7 +1,7 @@
 use std::{collections::VecDeque, fmt, future::ready, future::Future};
 
 use ntex::channel::{condition, oneshot, pool};
-use ntex::util::{ByteString, Bytes, Either, HashMap};
+use ntex::util::{ByteString, Bytes, Either, HashMap, PoolRef};
 use slab::Slab;
 
 use ntex_amqp_codec::protocol::{
@@ -281,6 +281,10 @@ impl SessionInner {
         self.id as u16
     }
 
+    pub(crate) fn memory_pool(&self) -> PoolRef {
+        self.sink.0.memory_pool()
+    }
+
     /// Set error. New operations will return error.
     pub(crate) fn set_error(&mut self, err: AmqpProtocolError) {
         log::trace!("Connection is failed, dropping state: {:?}", err);
